@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::SOAP11::Server;
 use vars '$VERSION';
-$VERSION = '2.01';
+$VERSION = '2.02';
 
 use base 'XML::Compile::SOAP11', 'XML::Compile::SOAP::Server';
 
@@ -23,9 +23,51 @@ sub init($)
     $self;
 }
 
-sub makeError()
-{   my $self = shift;
-    $self->faultWriter->(Fault => { @_ } );
+sub makeError(@)
+{   my ($self, %args) = @_;
+    info "Fault: $args{faultstring}";
+    $self->faultWriter->(Fault => \%args);
+}
+
+sub faultValidationFailed($$)
+{   my ($self, $name, $exception) = @_;
+
+    my $message =
+      __x"operation {name} for {version} called with invalid data"
+      , name => $name, version => 'SOAP11';
+
+    my $errors = XML::LibXML::Element->new('error');
+    $errors->appendText($exception->message->toString);
+    my $detail = XML::LibXML::Element->new('detail');
+    $detail->addChild($errors);
+
+    $self->makeError
+      ( faultcode   => pack_type(SOAP11ENV, 'Server.validationFailed')
+      , faultstring => $message
+      , faultactor  => $self->role
+      , detail      => $detail
+      );
+}
+
+sub faultResponseInvalid($$)
+{   my ($self, $name, $exception) = @_;
+
+    my $message  =
+      __x"procedure {name} for {version} produced an invalid response"
+       , name => $name, version => 'SOAP11';
+
+    my $errors = XML::LibXML::Element->new('error');
+    $errors->appendText($exception->message->toString);
+    my $detail = XML::LibXML::Element->new('detail');
+    $detail->addChild($errors);
+
+    # fault code does not really exist, but we need it.
+    $self->makeError
+      ( faultcode   => pack_type(SOAP11ENV, 'Server.invalidResponse')
+      , faultstring => $message
+      , faultactor  => $self->role
+      , detail      => $detail
+      );
 }
 
 sub faultNotImplemented($)
@@ -42,10 +84,12 @@ sub faultNotImplemented($)
       );
 }
 
-sub faultNoAnswerProduced()
-{   my ($self) = @_;
+sub faultNoAnswerProduced($)
+{   my ($self, $name) = @_;
     
-    my $message;
+    my $message =
+        __x"callback {name} did not return an answer", name => $name;
+
     $self->makeError
       ( faultcode   => pack_type(SOAP11ENV, 'Server.noAnswer')
       , faultstring => $message
